﻿using FileStorage;
using Microsoft.Extensions.Options;


namespace FileHandlers
{
     public class FwFileProvider:IFwFileProvider
    {
        private readonly IFileMetadataRepository _fileMetadataRepository;
        private readonly IOptionsSnapshot<FileUploadOptions> _fileUploadOptions;

        public FwFileProvider(IFileMetadataRepository fileMetadataRepository,IOptionsSnapshot<FileUploadOptions> fileUploadOptions)
        {
            _fileMetadataRepository = fileMetadataRepository;
            _fileUploadOptions = fileUploadOptions;
        }

        private IFwFileHandler  CreateFileHandler()
        {
            switch (_fileUploadOptions.Value.SaveFileMode)
            {
                case "Local":
                    return new FwLocalFileHandler(_fileUploadOptions);
                default:
                    return new FwLocalFileHandler(_fileUploadOptions);
            }
        }

        public IFileStorage Upload(string fileName, long length, string extraInfo, Stream fileData)
        {
            var fh = CreateFileHandler();
            if(fileName==null)
            {
                fileName = "unkown";
            }
            fileName = fileName.Replace("<", "").Replace(">", "").Replace(" ", "");

            var rv = fh.Upload(fileName, length, extraInfo,fileData);
            if(string.IsNullOrEmpty(rv))
            {
                FileMetadata fileModel = new FileMetadata();
                fileModel.FileName = fileName;
                var ext = string.Empty;
                if (string.IsNullOrEmpty(fileName) == false)
                {
                    var dotPos = fileName.LastIndexOf('.');
                    ext = fileName[(dotPos + 1)..];
                }
                fileModel.FileExt = ext;
                fileModel.Path = rv;
                fileModel.Length = length;
                fileModel.SaveMode = _fileUploadOptions.Value.SaveFileMode;
                fileModel.ExtraInfo = extraInfo;
                _fileMetadataRepository.AddAsync(fileModel);
                return fileModel;
            }
            return null;
        }
    }
}
