﻿using FileHandlers;
using FileStorage.Repository.SqlSugar.Extensions;
using FluentValidation;
using FluentValidation.AspNetCore;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.Extensions.FileProviders;
using Serilog;
using XF.Domain.Dependency;
using XF.Domain.Filters;
using XF.Domain.Options;

namespace FileStorage.Host;

internal static class StartupExtensions
{
    const string CorsOrigins = "CorsOrigins";
    internal static WebApplication ConfigureServices(this WebApplicationBuilder builder)
    {
        var services = builder.Services;
        var configuration = builder.Configuration;

        services.AddHttpContextAccessor();

#if DEBUG
        builder.WebHost.UseUrls("http://*:50102");
#endif

        services.Configure<StorageConfiguration>(d => configuration.GetSection(nameof(StorageConfiguration)).Bind(d));
        services.Configure<FileUploadOptions>(d => configuration.GetSection("FileUploadOptions").Bind(d));

        services.AddScoped<IFwFileProvider,FwFileProvider>();

        // Add services to the container.
        services.BatchInjectServices();

        //var storageConfig = configuration.GetSection(nameof(StorageConfiguration)).Get<StorageConfiguration>();
        //if(storageConfig == null || string.IsNullOrEmpty(storageConfig.Impt) || storageConfig.Impt == FileStorageConsts.LocalImpt)
        //{
            
        //}

        services.AddControllers(options =>
        {
            options.Filters.Add<UnifyResponseFilter>();
            options.Filters.Add<UserFriendlyExceptionFilter>();
        });

        // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
        services.AddEndpointsApiExplorer();

        //swagger
        services.RegisterSwagger();
        
        /* CORS */
        services.RegisterCors(configuration, CorsOrigins);

        //mapster
        services.RegisterMapper();

        ////mediatr
        //services.AddMediatR(d =>
        //{
        //    d.RegisterServicesFromAssembly(typeof(ApplicationStartupExtensions).Assembly);
        //});
        
        //sqlsugar
        services.AddSqlSugar(configuration);
        ////cache
        //services.AddCache(d =>
        //    {
        //        d.ConnectionString = configuration.GetConnectionString("Redis") ?? string.Empty;
        //        d.Prefix = "Hotline";
        //        d.TopicName = "hotline-topic";
        //    });

        ////validator
        //services.AddFluentValidationAutoValidation(config =>
        //{
        //    config.DisableDataAnnotationsValidation = true;
        //})
        //    .AddValidatorsFromAssembly(typeof(AppContractsStartupExtensions).Assembly);
        
        return builder.Build();
    }

    internal static WebApplication ConfigurePipelines(this WebApplication app)
    {
        app.UseSerilogRequestLogging();
        
        var swaggerEnable = app.Configuration.GetSection("Swagger").Get<bool>();
        // Configure the HTTP request pipeline.
        if (swaggerEnable)
        {
            app.UseSwagger();
            app.UseSwaggerUI();
        }
        app.UseCors(CorsOrigins);

        app.UseAuthentication();
        app.UseAuthorization();

        app.MapControllers();

        return app;
    }
}